#ifndef _EXCEPTION_H_
#define _EXCEPTION_H_

#include <string>

namespace cganim
  {
  /// Base class for exceptions generated by the library
  class Exception
    {
    public:
      
      /// Constructor
      Exception(const std::string& name = "")
        : m_name(name)
        {}
      
      /// Information about what happened
      virtual const std::string& What() const throw()
        { return m_name; }
      
    private:
      
      /// What happened string
      const std::string m_name;
    };
  
  /// Runtime logic exception: index of bounds, etc
  class LogicException: public Exception
    {
    public:

       LogicException(const std::string& name = "")
         : Exception(name)
         {
         }
    };



  /// Input/output exception
  class IOException: public Exception
    {
    public:

       IOException(const std::string& name = "")
         : Exception(name)
         {
         }
    };


   /// Invalid xml-file format
   class InvalidFileException: public IOException
    {
    public:

       InvalidFileException(const std::string& name = "")
         : IOException(name)
         {
         }
    };
  }
#endif
